#! /usr/bin/env ruby
#
# *******************************************
# 
#   Zone2MIDI - 2011 Tonality Systems Press
#
# *******************************************
#
# Written by Phil Legard (tonalitysystems@mac.com / phil@larkfall.co.uk)
#
# Use this program to add time signature changes to track 0 of a MIDI file
# to make your SCOM output MIDI files useable in notation and sequencing programs.
#
# Usage: ruby zone2midi <MIDI file> <text file> <output MIDI file>
#
# The text file should have zones in following format (as in SCOM syntax):
#
#		(2/4 2/4 5/8 4/2 4/4 ... )
#
# Example MIDI and zone-length files should be included in the archive
# that this script came with (Study1.mid and Study1.txt). An example of
# this file having been processed with Zone2MIDI is also included 
# (Study1-processed.mid) - try it in your notation package to check
# compatibility!
#
# This script requires Jim Menard's midlib library for Ruby, available at:
#
#		http://midilib.rubyforge.org/
#
# or install via Terminal: 
#
#		gem install midilib
#

$LOAD_PATH[0, 0] = File.join(File.dirname(__FILE__), '..', 'lib')

require 'midilib/sequence'
require 'midilib/consts'
include MIDI

puts "\nZone2MIDI\n2011 Tonality Systems Press (tonalitysystems@mac.com)\nWritten by Phil Legard (phil@larkfall.co.uk)\n"

# Command line arguments (MIDIProcess <midi file> <text file> <output name>)
if ARGV.length == 0
	puts "\nNo arguments supplied.\nUsage is: ruby zone2midi <midi file> <text file> <output file>\n\nAlternatively please enter them below - remember to include extensions!\n\n"
	puts "MIDI filename > "
	MIDIFilename = gets.chomp
	puts "Zone length filename > "
	TextFilename = gets.chomp
	puts "Output filename > "
	OutFilename = gets.chomp
else
 MIDIFilename = ARGV[0]
 TextFilename = ARGV[1]
 OutFilename = ARGV[2]
end

seq = MIDI::Sequence.new()

# Read MIDI file
File.open(MIDIFilename, 'rb') { | file |
    seq.read(file)
}

# Get track 0
t0 = seq.tracks[0]

# Delta time elapsed
dTime = 0

# Open time sigs file
TFile = File.open(TextFilename, "r")

TFile.each_line {|line| 
	theLine = line.dump
	theLine = theLine[2,theLine.length-4]
	theStrs = theLine.split(' ')

	theStrs.each { |theStrs|
		numerator = Integer(theStrs[0, theStrs.index('/')])
		denominator = theStrs[theStrs.index('/')+1, theStrs.length]

		# Special rules for handling zone lengths > time sigs:
		# If zone-length == 1/1 make it a 4/4
		if denominator==1
			if numerator==1
		 		numerator = 4
				denominator = "4"
		 	end
		end

		# Handle conversion of bottom number in time sig to MIDI-style denominator
		case denominator
		when "1"
			denPwr = 0
			addTime = seq.note_to_delta('whole')
			addTimeS = seq.note_to_length('whole')
		when "2"
			denPwr = 1
			addTime = seq.note_to_delta('half')
			addTimeS = seq.note_to_length('half')
		when "4"
			denPwr = 2
			addTime = seq.note_to_delta('quarter')
			addTimeS = seq.note_to_length('quarter')
		when "8"
			denPwr = 3
			addTime = seq.note_to_delta('eighth')
			addTimeS = seq.note_to_length('eighth')
		when "16"
			denPwr = 4
			addTime = seq.note_to_delta('sixteenth')
			addTimeS = seq.note_to_length('sixteenth')
		when "32"
			denPwr = 5
			addTime = seq.note_to_delta('thirtysecond')
			addTimeS = seq.note_to_length('thirtysecond')
		end

		# Standard MIDI settings 
		clocks = 24;
		qnotes = 8;

		# Create new TimeSig event
		theSig = TimeSig.new(numerator, denPwr, clocks, qnotes, 0)

		# Append delta time to new event
		theSig.delta_time = dTime

		# Time sig to events
		t0.events << theSig

		# Tidy up
		t0.recalc_times
		t0.recalc_delta_from_times

		# Increment delta time
		dTime = ((numerator * addTime))
	}
}

# Housekeeping
t0.sort()


File.open(OutFilename, 'wb') { | file |
	seq.write(file)
}

TFile.close

puts "\nSuccess!\n\n"
